/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.thermostat.qa.reporter;



import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;



/**
 * Log pages generator.
 * 
 * @author Pavel Tisnovsky <ptisnovs@redhat.com>
 */
public class LogPagesGenerator implements CommonGenerator
{
    private static final int TEST_NAME_PREFIX_LENGTH = "org.thermostat.qa.testsuites.".length();

    /**
     * 
     * @param testResults
     * @param params
     */
    public static void generate(Map<String, Map<String, List<String>>> testResults, CommandLineParameters params)
    {
        for (Entry<String, Map<String, List<String>>> testResult : testResults.entrySet())
        {
            createOneLogPage(testResult.getKey(), testResult.getValue(), params);
        }
    }

    /**
     * 
     * @param testDate
     * @param testResult
     * @param params
     */
    private static void createOneLogPage(String testDate, Map<String, List<String>> testResult,
                    CommandLineParameters params)
    {
        List<String> summaryLines = prepareSummaryLinesForLogPage(params);
        List<String> resultLines = prepareResultLinesForLogPage(testResult);
        List<String> template = FileUtils.readTextFile(params.getTemplateDir() + "/log.html");
        List<String> out = generateOutputForOneLogPage(testDate, summaryLines, resultLines, template);
        // write list of string to a file with given name
        File outputFile = new File(params.getReportDir(), "log_" + testDate + ".html");
        FileUtils.writeTextFile(outputFile, out);
    }

    /**
     * @param testDate
     * @param summaryLines
     * @param resultLines
     * @param template
     * @return
     */
    private static List<String> generateOutputForOneLogPage(String testDate, List<String> summaryLines,
                    List<String> resultLines, List<String> template)
    {
        List<String> out = new LinkedList<String>();
        // iterate through whole template
        for (String templateLine : template)
        {
            // replace text in template where needed
            if (templateLine.contains("${DATE}"))
            {
                templateLine = templateLine.replace("${DATE}", testDate);
                out.add(templateLine);
            }
            else if (templateLine.contains("${SUMMARY}"))
            {
                out.addAll(summaryLines);
            }
            else if (templateLine.contains("${RESULTS}"))
            {
                out.addAll(resultLines);
            }
            else
            {
                out.add(templateLine);
            }
        }
        return out;
    }

    /**
     * @return
     */
    private static List<String> prepareSummaryLinesForLogPage(CommandLineParameters params)
    {
        List<String> summaryLines = new LinkedList<String>();
        for (String testName : params.getTests())
        {
            summaryLines.add("<tr><td><a href='#" + testName + "'>" + testName + "</a></td></tr>");
        }
        return summaryLines;
    }
    
    /**
     * @param testResult
     * @return
     */
    private static List<String> prepareResultLinesForLogPage(Map<String, List<String>> testResult)
    {
        List<String> resultLines = new LinkedList<String>();
        for (Entry<String, List<String>> testSuite : testResult.entrySet())
        {
            String testSuiteName = getTestSuiteName(testSuite);
            System.out.println(testSuiteName);
            resultLines.add("<tr><td class='table-header' colspan='2'><a name='" + testSuiteName + "'></a>"
                            + testSuiteName + "</td></tr>");
            boolean failed = false;
            for (String testLine : testSuite.getValue())
            {
                if (testLine.startsWith("PASSED: "))
                {
                    if (failed)
                    {
                        resultLines.add("</pre></td><td>&nbsp;</td></tr>");
                        failed = false;
                    }
                    String testName = testLine.substring("PASSED: ".length());
                    String testPrefix = testName.substring(0, TEST_NAME_PREFIX_LENGTH);
                    String testPostfix = testName.substring(TEST_NAME_PREFIX_LENGTH);
                    resultLines.add("<tr><td><a name='" + testName + "'></a>" +
                                    "<span class='test-name-prefix'>" + testPrefix + "</span>" +
                                    "<span class='test-name-postfix'>" + testPostfix + "</span>" +
                                    "</td><td class='passed-text'>PASSED</td></tr>");
                }
                else if (testLine.startsWith("FAILED: "))
                {
                    String testName = testLine.substring("FAILED: ".length());
                    String aname = testName;
                    if (aname.indexOf(':') >=0) aname = aname.substring(0, aname.indexOf(':'));
                    resultLines.add("<tr><td class='failed-header'><a name='" + aname + "'></a>" + testName
                                    + "</td><td class='failed-text'>FAILED</td></tr>");
                    resultLines.add("<tr><td><pre class='stack-trace'>");
                    failed = true;
                }
                else if (testLine.startsWith("IGNORED: "))
                {
                    String testName = testLine.substring("IGNORED: ".length());
                    String aname = testName;
                    if (aname.indexOf(':') >=0) aname = aname.substring(0, aname.indexOf(':'));
                    resultLines.add("<tr><td class='ignored-header'><a name='" + aname + "'></a>" + testName
                                    + "</td><td class='ignored-text'>IGNORED</td></tr>");
                }
                else if (testLine.startsWith("SUMMARY: "))
                {
                    if (failed)
                    {
                        resultLines.add("</pre></td><td>&nbsp;</td></tr>");
                        failed = false;
                    }
                    testLine = StringUtils.replaceSpacesWithNbsps(testLine);
                    resultLines.add("<tr><td class='error-text' colspan='2'>" + testLine + "</td></tr>");
                }
                else if (failed)
                {
                    resultLines.add(testLine);
                }
            }
            resultLines.add("<tr><td colspan='2'>&nbsp;</td></tr>");
        }
        return resultLines;
    }

    /**
     * @param testSuite
     * @return
     */
    private static String getTestSuiteName(Entry<String, List<String>> testSuite)
    {
        String testSuiteName = testSuite.getKey();
        testSuiteName = testSuiteName.substring(testSuiteName.lastIndexOf('/') + 1, testSuiteName.lastIndexOf('.'));
        return testSuiteName;
    }
}
